/* Part of gjrand random number library version 4.0.1.0 or later. */
/* Copyright (C) 2004-2013 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

#include "gjrand.h"
#include "inte.h"
#include <math.h>
#include "nonstd.h"

int
gjrand_geometric (struct gjrand *s, double p)
{
    double t, u;
    uint64_t a, b, c, d;
    int r;

    if (p>=1) return 1;
    if (p<=0) return 0x7fffffff;

    GJRAND_INTE_LOADSTATE(s, a, b, c, d);
    GJRAND_INTE_CRANK(a, b, c, d);
    GJRAND_INTE_STORESTATE(s, a, b, c, d);

    t = ((double)((a>>11)|1)) * S53;

    if (p>3.0/64)
    {
	p = 1.0-p;
	u = 1.0;
	r = 0;
	do
	{
	    u *= p;
	    r++;
	} while (u>t);
	return r;
    }

    r = (int)(log(t)/log1p(-p)) + 1;

    return r;
}
