/* Part of gjrand random number library version 3.3.3.0 or later. */
/* Copyright (C) 2004-2010 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

#include "gjrand.h"
#include "inte.h"
#include <sys/time.h>
#include <unistd.h>
#ifdef USE_POSIX_THREADS
#include <pthread.h>
#endif

/* We try fairly hard to make this do a different seeding for two different */
/* calls. Firstly we fold in the pid so different processes get different */
/* seeds. Secondly if threads are enabled, the thread id is also used. */
/* We also use the seconds and microseconds time values. Unfortunately */
/* this is not always enough: some computers only update the microseconds */
/* field at fairly slow intervals, for instance 0.01 seconds was once */
/* popular on linux. And even with update every microsecond, the time is */
/* coming soon where a computer will be quick enough to do two calls */
/* to this routine in less than a microsecond. To beat even this, */
/* nonce is used, which should give a different seed for subsequent calls */
/* from within the same process, even if the time hasn't changed. This may */
/* appear to not be threadsafe, and perhaps by the strictest definitions */
/* it isn't. Trust me though, unless you are doing some maximum security */
/* audit: in most settings it is safe even in multi-threaded code. */

/* This uses POSIX system calls in an attempt to get highly variable */
/* information out of the operating system. Linux, BSD, commercial Unix, */
/* and MAC OS starting at version 10 should have this stuff. */
/* Android has Linux hiding somewhere underneath and should also work. */
/* I'm not so sure about MS-Windows. If these calls aren't provided */
/* as library functions, just substitute WIN32 or whatever calls */
/* that do approximately the same thing, or at least provide some */
/* sort of pseudo-unpredictable system data. Then, ideally, do some */
/* #ifdef work and post on gjrand forums so i can include the fix in */
/* later versions. */

void
gjrand_initrand(struct gjrand *s)
{
    struct timeval t;
    static uint32_t nonce=0;

    gettimeofday(&t, 0); /* seconds and microseconds timer */
    s->a = (((uint64_t)(t.tv_sec))<<32) | (uint32_t)(t.tv_usec);

#ifdef USE_POSIX_THREADS
    s->b=pthread_self(); /* thread id, if available */
#else
    s->b=getppid(); /* otherwise something else */
#endif

    s->b |= ((uint64_t)(getpid()))<<32; /* process id */

    nonce += 1397u;
    s->c = (((uint64_t)nonce)<<32)|4000001u;
    s->d = 0;

    gjrand_inte_mixstate(s);
}
