/* Part of gjrand random number library version 3.4.0.0 or later. */
/* Copyright (C) 2004-2011 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

#include <string.h>

#include "gjrand.h"
#include "inte.h"

/* High speed method for large vectors of normal variates, partly based */
/* on the ideas of Christopher Stewart Wallace as in FastNorm etc. */

static void
startnorm(struct gjrand *g, double *b)
{
    double *bend=b+256, d0, d1;
    uint64_t w, x, y, z;
    uint32_t w0, w1;

    GJRAND_INTE_LOADSTATE(g, w, x, y, z);

    do
    {
	GJRAND_INTE_CRANK(w, x, y, z);
	w0 = (uint32_t)w;
	w1 = (uint32_t)(w>>32);

	b+=8;

	d0 = gjrand_inte_normal_app[w0&255];
	d1 = gjrand_inte_normal_app[w1&255];
	b[-8] = d0; b[-7] = d1;

	d0 = gjrand_inte_normal_app[(w0>>8)&255];
	d1 = gjrand_inte_normal_app[(w1>>8)&255];
	b[-6] = d0; b[-5] = d1;

	d0 = gjrand_inte_normal_app[(w0>>16)&255];
	d1 = gjrand_inte_normal_app[(w1>>16)&255];
	b[-4] = d0; b[-3] = d1;

	d0 = gjrand_inte_normal_app[(w0>>24)];
	d1 = gjrand_inte_normal_app[(w1>>24)];
	b[-2] = d0; b[-1] = d1;
    } while (b<bend);

    GJRAND_INTE_STORESTATE(g, w, x, y, z);
}

static void
impnorm(double *bin, double *bout)
{
    double *bend=bout+256, t, a, b, c, d;

    do
    {
	a = bin[0]; b = bin[64];
	c = bin[128]; d = bin[192];
	t = ((a+b)+(c+d))*0.5;
	bout += 4;
	a -= t; b -= t; c = t-c; d = t-d;
	bin++;
	bout[-4] = a; bout[-3] = b; bout[-2] = c; bout[-1] = d;
    } while (bout<bend);
}

static void
gjrand_inte_normalvb(struct gjrand *g, int n, double *v)
{
    double b[256], t[256], *p=v;

    while (n>0)
    {
	if (n<256) p = t;
	startnorm(g, p);
	impnorm(p, b);
	impnorm(b, p);
	impnorm(p, b);
	impnorm(b, p);
	if (n<256) {memcpy(v, p, n*sizeof(double)); return;}
	p += 256;
	v += 256;
	n -= 256;
    }
}

void
gjrand_normalv(struct gjrand *s, int n, double *v)
{
    if (n<45) gjrand_inte_normalvs(s, n, v);
    else gjrand_inte_normalvb(s, n, v);
}
