/* Miscellaneous stuff for gjrand random numbers version 3.2.1.0 or later. */
/* Copyright (C) 2004-2010 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

#ifndef _GJRAND_BINOM_H
#define _GJRAND_BINOM_H 1

#ifndef _GJRAND_INTEG2_H
#include "integ2.h"
#endif

#ifndef _GJRAND_MY_INT_TYPES_H
#include "../src/my_int_types.h"
#endif

#include "lgamma.h"

static double save_x, logfac_x, log_p, log_1mp;

static void
initxp(double x, double p)
{
    save_x = x;
    logfac_x = logfac(x);
    log_p = log(p);
    log_1mp = log(1.0-p);
}

static double
bino(double y)
{
    return exp(logfac_x - logfac(save_x-y) - logfac(y) +
		log_1mp*(save_x-y) + log_p*y);
}

/* what is the probability that throwing x coins (probability p of heads) */
/* you get a result further from expected than y? */
/* Both surprisingly low and high y makes a small P-value. */
/* Not very accurate? */
static double
sumbino(int64_t x, int64_t y, double p)
{
    double r=0.0, z, incr, tiny;
    int64_t j;

    if (p<=0.0) {if (y<=0) r = 1.0; goto ret;}
    if (p>=1.0) {if (y>=x) r = 1.0; goto ret;}
    if (y<0) goto ret;
    if (y>x) goto ret;

    z = p*x;
    if (y>z) {z = x-z; y = x-y; p = 1.0-p;}

    initxp(x, p);

    if (2.0*z>x) z = x-z;
    incr = sqrt(z);

    if (incr>60.0) r = gjrt_integ2(bino, y, -incr, 1.0e-5, 4) * 2.0;
    else
    {
	r = bino((double)y);
	tiny = r * 1.0e-6;
	for (j=y-1; j>=0; j--)
	{
	    z = bino((double)j);
	    r += z*2.0;
	    if (z < tiny) break;
	}
    }

    ret:
    if (r>1.0) r = 2.0-r;
    return r;
}

#endif /* _GJRAND_BINOM_H */
