/* Miscellaneous stuff for gjrand random numbers version 4.0.1.0 or later. */
/* Copyright (C) 2004-2013 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

/* Experimental test that might replace chisquared with moderately large df */
/* under certain conditions. */
/* 1. number of buckets is a power of 2. (ie df+1). */
/* 2. expectation value for each bucket is the same. */
/* 3. bucket index has at least ordinal value */
/*    (although a cyclic system where highest also adjacent to lowest is ok.) */
/* Waiting for testing to see if it is really more powerful. */

/* btreetest() is the useful function to be called from outside. */

#ifndef _GJRAND_BTREE_H
#define _GJRAND_BTREE_H 1

#include "binom.h"
#include "pscale.h"

static double
btree2(const int64_t buckets[], int chunk)
{
    int64_t left=0, tot;
    int half=chunk>>1, j;

    if ((chunk&1)!=0) return -1.0;

    for (j=0; j<half; j++) left += buckets[j];
    tot = left;
    for (j=half; j<chunk; j++) tot += buckets[j];
    if (tot<15) return -1.0;

    return sumbino(tot, left, 0.5);
}

static double
btree1(const int64_t buckets[], int n, int chunk)
{
    double plo = 2.0, x;
    int j, ct=0;

    for (j=0; j<n; j += chunk)
    {
	x = btree2(buckets+j, chunk);
	if (x>=0.0)
	{
	    if (x<plo) plo = x;
	    ct++;
	}
    }

    if (ct==0) return -1.0;

    return pco_scale(plo, (double)ct);
}

/* returns a P-value */
static double
btreetest(const int64_t buckets[], int n)
{
    double plo = 2.0, x;
    int j, ct=0;

    for (j=n; j>1; j>>=1)
    {
	x = btree1(buckets, n, j);
	if (x<0.0) break;
	if (x<plo) plo = x;
	ct++;
    }

    if (ct==0) return -1.0;

    return pco_scale(plo, (double)ct);
}

#endif /* _GJRAND_BTREE_H */
