/* Miscellaneous stuff for gjrand random numbers version 3.3.2.0 or later. */
/* Copyright (C) 2004-2010 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "../../src/my_int_types.h"

/* This one writes data for use by the tests in testunif */

/* second optional arg is random seed. */
/* This attempts to write forever. */

/* This is a lagged Fibonacci generator, or additive feedback shift register. */
/* A more general lagged Fibonacci generator can */
/* multiply feedback taps by constants as well. Note that the least */
/* significant bits here are a lfsr with period 65535. All the other */
/* bits are more complicated because of the carry bits that feed in. */
/* The period of the whole generator is apparently very long. */

/* This one is based on a "pentanomial", which should have slightly */
/* better properties than the "trinomials" that are usually favoured. */
/* On the other hand trinomials are faster. Sometimes extremely long lags */
/* are recommended, and for the most part those do have better properties */
/* than short-lag generators like this one. */

static void
update(uint32_t x[16])
{
    int i;

    for (i=0; i<16; i++) x[i] += x[(i-5)&15]+x[(i-3)&15]+x[(i-2)&15];
}

static void
blat(uint32_t seed)
{
    int i;
    uint32_t buf[16];

    buf[0]=seed;
    buf[1]= 0xffffffff;
    buf[2]=987654321;
    for (i=3; i<16; i++) buf[i]=buf[i-1]+123456789;
    update(buf);
    buf[0]^=buf[0]>>16;
    buf[1]^=buf[1]>>24;
    buf[2]^=buf[2]>>28;
    buf[3]^=buf[3]>>30;
    buf[4]^=buf[4]>>31;
    update(buf);

    while (1)
    {
	update(buf);
	if (fwrite(buf, 1, 64, stdout)!=64) break;
    }
}

int
main(int argc, char **argv)
{
    unsigned long seed;

    if (argc>1) sscanf(argv[1], "%lu", &seed);
    else seed = time(0);

    blat(seed);

    return 0;
}
