/* Various stuff included with gjrand random numbers version 3.3.0.0 . */
/* Copyright (C) 2004-2009 G. Jones. */
/* Licensed under the GNU General Public License version 2 or 3. */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "../src/gjrand.h"

#define MULT (1017.0/1024.0)

double
adj(void)
{
    int j;
    long double t, m;

    t = 0; m = 1;
    for (j=0; j<16; j++)
    {
	t += m*m;
	m *= MULT;
    }

    t = sqrtl(1/t);
 printf("#define GJRAND_INTE_NORM_ADJ (%.18Lf)\n\n", t);
    return m;
}

double
powi(double x, int p)
{
    double r=1.0;
    while (p>0) {r*=x; p--;}
    return r;
}

void
moments3(double x[256])
{
    double y[256];
    double s=0.0, p=1.0, p2;
    int i, j, k;

    for (i=0; i<256; i++) for (j=0; j<256; j++) for (k=0; k<256; k++)
	s+=(x[i]+x[j]+x[k])*(x[i]+x[j]+x[k]);
    s/=256.0*256.0*256.0;
    s=sqrt(s);
    s=1.0/s;
    for (i=0; i<256; i++) y[i]=x[i]*s;

    for (i=4; i<=14; i+=2)
    {
	int n;
	p2=0.0;
	for (j=0; j<256; j++) for (k=0; k<256; k++) for (n=0; n<256; n++)
	    p2+=powi((y[j]+y[k]+y[n]), i);
	p2/=256.0*256.0*256.0;
printf("%12.8f ",p2/p);
	p=p2;
    }
    fputs("\n", stdout);
}

void
moments2(double x[256])
{
    double y[256];
    double s=0.0, p=1.0, p2;
    int i, j;

    for (i=0; i<256; i++) for (j=0; j<256; j++) s+=(x[i]+x[j])*(x[i]+x[j]);
    s/=65536.0;
    s=sqrt(s);
    s=1.0/s;
    for (i=0; i<256; i++) y[i]=x[i]*s;

    for (i=4; i<=14; i+=2)
    {
	int k;
	p2=0.0;
	for (j=0; j<256; j++) for (k=0; k<256; k++) p2+=powi((y[j]+y[k]), i);
	p2/=65536.0;
printf("%12.8f ",p2/p);
	p=p2;
    }
    fputs("\n", stdout);
}

void
moments(double x[256])
{
    int i;
    long double s=0.0, p=1.0;
    for (i=0; i<256; i++) s+=x[i]*x[i];
    s/=256.0;
    s=sqrtl(s);
    for (i=0; i<256; i++) x[i]/=s;

#if 1
    for (i=4; i<=14; i+=2)
    {
	int j;
	s=0.0;
	for (j=0; j<256; j++) s+=powi(x[j], i);
	s/=256.0;
printf("%12.8Lf ",s/p);
	p=s;
    }
    fputs("\n", stdout);

    moments2(x);
    moments3(x);
#endif
}

#define MEG (512*1024)
double g[8*MEG+1], gi[4*MEG];

void
makgi(void)
{
    int i, j;
    double d,s;

// printf("g[]\n");
    for (i=0; i<8*MEG+1; i++)
    {
	double d=(double)i/(double)MEG;
	g[i]=exp(-d*d);
// printf("%f\n", g[i]);
    }

    s=0.0;
    for (i=0; i<4*MEG; i++)
    {
	j=8*MEG-2-2*i;
	d=g[j]+g[j+2]+4.0*g[j+1];
	s+=d;
	gi[i]=s;
    }

    for (i=0; i<4*MEG; i++) gi[i]/=s;

// printf("\ngi[]\n");
// for (i=0; i<4*MEG; i++)
// printf("%f\n", gi[i]);
// printf("\n");
}

void
maktab(double tab[256])
{
    // Geronimo's special fudge factors. These improve some higher moments.
    static const double fud4=0.29265889, fud6= -0.495165, fud8=0.235065;

    int i, k;

    k=0;
    for (i=0; i<128; i++)
    {
	double d=(double)(2*i+2)/256.0, e=0.0, f=0.0;
	int j=k;
	if (i==127) k=4*MEG;
	else while (gi[k]<d) k++;

	while (j<k)
	{
	    e+=gi[j];
	    f+=(double)(4*MEG-1-j)/(double)MEG*gi[j];
	    j++;
	}

	tab[i]=f/e;
    }

    for (i=0; i<128; i++)
    {
	double y=tab[i];
	double z=y*y*y;
	tab[i]+=z*4*fud4;
	z*=y*y;
	tab[i]+=z*6*fud6;
	z*=y*y;
	tab[i]+=z*8*fud8;
    }

    for (i=0; i<128; i++) tab[255-i]= -tab[i];
}

/* Note measures to remove linear correlation between i and x[i] */
void
scramble(double x[256])
{
    int i, j;
    struct gjrand g;
    double s,t;

    gjrand_init(&g, 42);

    for (i=1; i<128; i++)
    {
	j=gjrand_rand32mod(&g, i+1);
	t=x[i]; x[i]=x[j]; x[j]=t;
    }

    s=0.0;
    for (i=127; i>=0; i--)
    {
	t=x[i];
	if (s>=0) {s-=t; x[2*i]= -t; x[2*i+1]=t;}
	else {s+=t; x[2*i]=t; x[2*i+1]= -t;}
    }
}

void
printa(double x[256], double a)
{
    int i;
    printf("static const double gjrand_inte_normal_app[256]={\n");
    for (i=0; i<255; i++) printf("%.17f,\n", x[i]*a);
    printf("%.17f};\n", x[255]*a);
}

int
main(int argc, char **argv)
{
    double tab[256];

    (void)adj();
    makgi();
    maktab(tab);

    printf("#if 0\n/* moments */\n");
    moments(tab);

    printf("/* unscrambled */\n");
    printa(tab, 1.0);
    printf("#endif\n");

    scramble(tab);

    printa(tab, 1.0);

    return 0;
}
